@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2000, 2014, Oracle and/or its affiliates.
-   Copyright (c) 2010, 2017, Monty Program Ab.
+   Copyright (c) 2010, 2017, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
 
 /* maintaince of mysql databases */
 
@@ -23,9 +23,10 @@
 #include <sys/stat.h>
 #include <mysql.h>
 #include <sql_common.h>
-#include <welcome_copyright_notice.h>           /* ORACLE_WELCOME_COPYRIGHT_NOTICE */
+#include <welcome_copyright_notice.h>
+#include <my_rnd.h>
 
-#define ADMIN_VERSION "9.0"
+#define ADMIN_VERSION "9.1"
 #define MAX_MYSQL_VAR 512
 #define SHUTDOWN_DEF_TIMEOUT 3600		/* Wait for shutdown */
 #define MAX_TRUNC_LENGTH 3
@@ -37,8 +38,8 @@ char ex_var_names[MAX_MYSQL_VAR][FN_REFLEN];
 ulonglong last_values[MAX_MYSQL_VAR];
 static int interval=0;
 static my_bool option_force=0,interrupted=0,new_line=0,
-               opt_compress=0, opt_relative=0, opt_verbose=0, opt_vertical=0,
-               tty_password= 0, opt_nobeep;
+               opt_compress= 0, opt_local= 0, opt_relative= 0, opt_verbose= 0,
+               opt_vertical= 0, tty_password= 0, opt_nobeep;
 static my_bool debug_info_flag= 0, debug_check_flag= 0;
 static uint tcp_port = 0, option_wait = 0, option_silent=0, nr_iterations;
 static uint opt_count_iterations= 0, my_end_arg;
@@ -100,9 +101,13 @@ enum commands {
   ADMIN_FLUSH_HOSTS,      ADMIN_FLUSH_TABLES,    ADMIN_PASSWORD,
   ADMIN_PING,             ADMIN_EXTENDED_STATUS, ADMIN_FLUSH_STATUS,
   ADMIN_FLUSH_PRIVILEGES, ADMIN_START_SLAVE,     ADMIN_STOP_SLAVE,
-  ADMIN_FLUSH_THREADS,    ADMIN_OLD_PASSWORD,    ADMIN_FLUSH_SLOW_LOG,
+  ADMIN_START_ALL_SLAVES, ADMIN_STOP_ALL_SLAVES,
+  ADMIN_FLUSH_THREADS,    ADMIN_OLD_PASSWORD,    ADMIN_FLUSH_BINARY_LOG,
+  ADMIN_FLUSH_ENGINE_LOG, ADMIN_FLUSH_ERROR_LOG, ADMIN_FLUSH_GENERAL_LOG,
+  ADMIN_FLUSH_RELAY_LOG,  ADMIN_FLUSH_SLOW_LOG,
   ADMIN_FLUSH_TABLE_STATISTICS, ADMIN_FLUSH_INDEX_STATISTICS,
   ADMIN_FLUSH_USER_STATISTICS, ADMIN_FLUSH_CLIENT_STATISTICS,
+  ADMIN_FLUSH_USER_RESOURCES,
   ADMIN_FLUSH_ALL_STATUS, ADMIN_FLUSH_ALL_STATISTICS
 };
 static const char *command_names[]= {
@@ -113,9 +118,11 @@ static const char *command_names[]= {
   "flush-hosts",          "flush-tables",        "password",
   "ping",                 "extended-status",     "flush-status",
   "flush-privileges",     "start-slave",         "stop-slave",
-  "flush-threads", "old-password", "flush-slow-log",
+  "start-all-slaves", "stop-all-slaves",
+  "flush-threads", "old-password", "flush-binary-log", "flush-engine-log",
+  "flush-error-log", "flush-general-log", "flush-relay-log", "flush-slow-log",
   "flush-table-statistics", "flush-index-statistics",
-  "flush-user-statistics", "flush-client-statistics",
+  "flush-user-statistics", "flush-client-statistics", "flush-user-resources",
   "flush-all-status", "flush-all-statistics",
   NullS
 };
@@ -157,6 +164,9 @@ static struct my_option my_long_options[] =
    NO_ARG, 0, 0, 0, 0, 0, 0},
   {"host", 'h', "Connect to host.", &host, &host, 0, GET_STR,
    REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+  {"local", 'l', "Local command, don't write to binlog.",
+   &opt_local, &opt_local, 0, GET_BOOL, NO_ARG, 0, 0, 0,
+   0, 0, 0},
   {"no-beep", 'b', "Turn off beep on error.", &opt_nobeep,
    &opt_nobeep, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0}, 
   {"password", 'p',
@@ -341,8 +351,12 @@ int main(int argc,char *argv[])
   }
 #ifdef HAVE_OPENSSL
   if (opt_use_ssl)
+  {
     mysql_ssl_set(&mysql, opt_ssl_key, opt_ssl_cert, opt_ssl_ca,
 		  opt_ssl_capath, opt_ssl_cipher);
+    mysql_options(&mysql, MYSQL_OPT_SSL_CRL, opt_ssl_crl);
+    mysql_options(&mysql, MYSQL_OPT_SSL_CRLPATH, opt_ssl_crlpath);
+  }
   mysql_options(&mysql,MYSQL_OPT_SSL_VERIFY_SERVER_CERT,
                 (char*)&opt_ssl_verify_server_cert);
 #endif
@@ -361,6 +375,9 @@ int main(int argc,char *argv[])
   if (opt_default_auth && *opt_default_auth)
     mysql_options(&mysql, MYSQL_DEFAULT_AUTH, opt_default_auth);
 
+  mysql_options(&mysql, MYSQL_OPT_CONNECT_ATTR_RESET, 0);
+  mysql_options4(&mysql, MYSQL_OPT_CONNECT_ATTR_ADD,
+                 "program_name", "mysqladmin");
   if (sql_connect(&mysql, option_wait))
   {
     /*
@@ -597,6 +614,7 @@ static my_bool sql_connect(MYSQL *mysql, uint wait)
 
 static int execute_commands(MYSQL *mysql,int argc, char **argv)
 {
+  int ret = 0;
   const char *status;
   /*
     MySQL documentation relies on the fact that mysqladmin will
@@ -607,6 +625,18 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
   */
 
   struct my_rnd_struct rand_st;
+  char buff[FN_REFLEN + 20];
+
+  if (opt_local)
+  {
+    sprintf(buff, "set local sql_log_bin=0");
+    if (mysql_query(mysql, buff))
+    {
+      my_printf_error(0, "SET LOCAL SQL_LOG_BIN=0 failed; error: '%-.200s'",
+                      error_flags, mysql_error(mysql));
+      return -1;
+    }
+  }
 
   for (; argc > 0 ; argv++,argc--)
   {
@@ -614,7 +644,6 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     switch ((command= find_type(argv[0],&command_typelib,FIND_TYPE_BASIC))) {
     case ADMIN_CREATE:
     {
-      char buff[FN_REFLEN+20];
       if (argc < 2)
       {
 	my_printf_error(0, "Too few arguments to create", error_flags);
@@ -892,6 +921,56 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       }
       break;
     }
+    case ADMIN_FLUSH_BINARY_LOG:
+    {
+      if (mysql_query(mysql, "flush binary logs"))
+      {
+        my_printf_error(0, "flush failed; error: '%s'", error_flags,
+                        mysql_error(mysql));
+        return -1;
+      }
+      break;
+    }
+    case ADMIN_FLUSH_ENGINE_LOG:
+    {
+      if (mysql_query(mysql,"flush engine logs"))
+      {
+        my_printf_error(0, "flush failed; error: '%s'", error_flags,
+                        mysql_error(mysql));
+        return -1;
+      }
+      break;
+    }
+    case ADMIN_FLUSH_ERROR_LOG:
+    {
+      if (mysql_query(mysql, "flush error logs"))
+      {
+        my_printf_error(0, "flush failed; error: '%s'", error_flags,
+                        mysql_error(mysql));
+        return -1;
+      }
+      break;
+    }
+    case ADMIN_FLUSH_GENERAL_LOG:
+    {
+      if (mysql_query(mysql, "flush general logs"))
+      {
+        my_printf_error(0, "flush failed; error: '%s'", error_flags,
+                        mysql_error(mysql));
+        return -1;
+      }
+      break;
+    }
+    case ADMIN_FLUSH_RELAY_LOG:
+    {
+      if (mysql_query(mysql, "flush relay logs"))
+      {
+        my_printf_error(0, "flush failed; error: '%s'", error_flags,
+                        mysql_error(mysql));
+        return -1;
+      }
+      break;
+    }
     case ADMIN_FLUSH_SLOW_LOG:
     {
       if (mysql_query(mysql,"flush slow logs"))
@@ -962,6 +1041,16 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       }
       break;
     }
+    case ADMIN_FLUSH_USER_RESOURCES:
+    {
+      if (mysql_query(mysql, "flush user_resources"))
+      {
+        my_printf_error(0, "flush failed; error: '%s'", error_flags,
+                        mysql_error(mysql));
+        return -1;
+      }
+      break;
+    }
     case ADMIN_FLUSH_CLIENT_STATISTICS:
     {
       if (mysql_query(mysql,"flush client_statistics"))
@@ -1019,7 +1108,8 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
         if (strcmp(typed_password, verified) != 0)
         {
           my_printf_error(0,"Passwords don't match",MYF(ME_BELL));
-          return -1;
+          ret = -1;
+          goto password_done;
         }
       }
       else
@@ -1046,7 +1136,8 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
           {
             my_printf_error(0, "Could not determine old_passwords setting from server; error: '%s'",
                 	    error_flags, mysql_error(mysql));
-            return -1;
+            ret = -1;
+            goto password_done;
           }
           else
           {
@@ -1057,7 +1148,8 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
                               "Could not get old_passwords setting from "
                               "server; error: '%s'",
         		      error_flags, mysql_error(mysql));
-              return -1;
+              ret = -1;
+              goto password_done;
             }
             if (!mysql_num_rows(res))
               old= 1;
@@ -1082,15 +1174,15 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       {
 	my_printf_error(0, "Can't turn off logging; error: '%s'",
 			error_flags, mysql_error(mysql));
-	return -1;
+        ret = -1;
       }
+      else
       if (mysql_query(mysql,buff))
       {
 	if (mysql_errno(mysql)!=1290)
 	{
 	  my_printf_error(0,"unable to change password; error: '%s'",
 			  error_flags, mysql_error(mysql));
-	  return -1;
 	}
 	else
 	{
@@ -1104,9 +1196,10 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
 			  " --skip-grant-tables).\n"
 			  "Use: \"mysqladmin flush-privileges password '*'\""
 			  " instead", error_flags);
-	  return -1;
 	}
+        ret = -1;
       }
+password_done:
       /* free up memory from prompted password */
       if (typed_password != argv[1]) 
       {
@@ -1118,26 +1211,67 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
     }
 
     case ADMIN_START_SLAVE:
-      if (mysql_query(mysql, "START SLAVE"))
+    case ADMIN_START_ALL_SLAVES:
+    {
+      my_bool many_slaves= 0;
+      const char *query= "START SLAVE";
+      if (command == ADMIN_START_ALL_SLAVES && mariadb_connection(mysql) &&
+          mysql_get_server_version(mysql) >= 100000)
+      {
+        query="START ALL SLAVES";
+        many_slaves= 1;
+      }
+
+      if (mysql_query(mysql, query))
       {
 	my_printf_error(0, "Error starting slave: %s", error_flags,
 			mysql_error(mysql));
 	return -1;
       }
+      else if (!many_slaves || mysql_warning_count(mysql) > 0)
+      {
+        if (!option_silent)
+          puts("Slave('s) started");
+      }
       else
-	puts("Slave started");
+      {
+        if (!option_silent)
+          puts("No slaves to start");
+      }
       break;
+    }
     case ADMIN_STOP_SLAVE:
-      if (mysql_query(mysql, "STOP SLAVE"))
+    case ADMIN_STOP_ALL_SLAVES:
+    {
+      const char *query= "STOP SLAVE";
+      my_bool many_slaves= 0;
+
+      if (command == ADMIN_STOP_ALL_SLAVES && mariadb_connection(mysql) &&
+          mysql_get_server_version(mysql) >= 100000)
+      {
+        query="STOP ALL SLAVES";
+        many_slaves= 1;
+      }
+
+      if (mysql_query(mysql, query))
       {
 	  my_printf_error(0, "Error stopping slave: %s", error_flags,
 			  mysql_error(mysql));
 	  return -1;
       }
+      else if (!many_slaves || mysql_warning_count(mysql) > 0)
+      {
+        /* We can't detect if there was any slaves to stop with STOP SLAVE */
+        if (many_slaves && !option_silent)
+          puts("Slave('s) stopped");
+      }
       else
-	puts("Slave stopped");
+      {
+        if (!option_silent)
+          puts("All slaves was already stopped");
+      }
       break;
-
+    }
     case ADMIN_PING:
       mysql->reconnect=0;	/* We want to know of reconnects */
       if (!mysql_ping(mysql))
@@ -1167,7 +1301,7 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       return 1;
     }
   }
-  return 0;
+  return ret;
 }
 
 /**
@@ -1244,12 +1378,18 @@ static void usage(void)
   flush-index-statistics  Flush index statistics\n\
   flush-logs              Flush all logs\n\
   flush-privileges        Reload grant tables (same as reload)\n\
+  flush-binary-log        Flush binary log\n\
+  flush-engine-log        Flush engine log(s)\n\
+  flush-error-log         Flush error log\n\
+  flush-general-log       Flush general log\n\
+  flush-relay-log         Flush relay log\n\
   flush-slow-log          Flush slow query log\n\
-  flush-status		  Clear status variables\n\
+  flush-status            Clear status variables\n\
   flush-table-statistics  Clear table statistics\n\
   flush-tables            Flush all tables\n\
   flush-threads           Flush the thread cache\n\
   flush-user-statistics   Flush user statistics\n\
+  flush-user-resources    Flush user resources\n\
   kill id,id,...	Kill mysql threads");
 #if MYSQL_VERSION_ID >= 32200
   puts("\
