@@ -1,4 +1,5 @@
 /* Copyright (C) 2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
+   Copyright (c) 2011, 2013, Monty Program Ab.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 /**
   @file
@@ -403,7 +404,7 @@ static void wt_resource_init(uchar *arg)
   bzero(rc, sizeof(*rc));
   rc_rwlock_init(rc);
   mysql_cond_init(key_WT_RESOURCE_cond, &rc->cond, 0);
-  my_init_dynamic_array(&rc->owners, sizeof(WT_THD *), 0, 5);
+  my_init_dynamic_array(&rc->owners, sizeof(WT_THD *), 0, 5, MYF(0));
   DBUG_VOID_RETURN;
 }
 
@@ -510,7 +511,7 @@ void wt_thd_lazy_init(WT_THD *thd, const ulong *ds, const ulong *ts,
   thd->deadlock_search_depth_long= dl;
   thd->timeout_long= tl;
   /* dynamic array is also initialized lazily - without memory allocations */
-  my_init_dynamic_array(&thd->my_resources, sizeof(WT_RESOURCE *), 0, 5);
+  my_init_dynamic_array(&thd->my_resources, sizeof(WT_RESOURCE *), 0, 5, MYF(0));
 #ifndef DBUG_OFF
   thd->name= my_thread_name();
 #endif
@@ -560,7 +561,7 @@ my_bool wt_resource_id_memcmp(const void *a, const void *b)
 {
   /* we use the fact that there's no padding in the middle of WT_RESOURCE_ID */
   compile_time_assert(offsetof(WT_RESOURCE_ID, type) == sizeof(ulonglong));
-  return memcmp(a, b, sizeof_WT_RESOURCE_ID);
+  return MY_TEST(memcmp(a, b, sizeof_WT_RESOURCE_ID));
 }
 
 /**
@@ -604,8 +605,6 @@ static int deadlock_search(struct deadlock_arg *arg, WT_THD *blocker,
   DBUG_PRINT("wt", ("enter: thd=%s, blocker=%s, depth=%u",
                     arg->thd->name, blocker->name, depth));
 
-  LF_REQUIRE_PINS(1);
-
   arg->last_locked_rc= 0;
 
   if (depth > arg->max_depth)
@@ -923,8 +922,6 @@ int wt_thd_will_wait_for(WT_THD *thd, WT_THD *blocker,
   WT_RESOURCE *rc;
   DBUG_ENTER("wt_thd_will_wait_for");
 
-  LF_REQUIRE_PINS(3);
-
   DBUG_PRINT("wt", ("enter: thd=%s, blocker=%s, resid=%lu",
                     thd->name, blocker->name, (ulong)resid->value));
 
@@ -1069,7 +1066,7 @@ int wt_thd_cond_timedwait(WT_THD *thd, mysql_mutex_t *mutex)
     ret= WT_OK;
   rc_unlock(rc);
 
-  end_wait_time= starttime.val *1000 + (*thd->timeout_short)*ULL(1000000);
+  end_wait_time= starttime.val *1000 + (*thd->timeout_short)*1000000ULL;
   set_timespec_time_nsec(timeout, end_wait_time);
   if (ret == WT_TIMEOUT && !thd->killed)
     ret= mysql_cond_timedwait(&rc->cond, mutex, &timeout);
@@ -1082,7 +1079,7 @@ int wt_thd_cond_timedwait(WT_THD *thd, mysql_mutex_t *mutex)
       ret= WT_DEADLOCK;
     else if (*thd->timeout_long > *thd->timeout_short)
     {
-      end_wait_time= starttime.val *1000 + (*thd->timeout_long)*ULL(1000000);
+      end_wait_time= starttime.val *1000 + (*thd->timeout_long)*1000000ULL;
       set_timespec_time_nsec(timeout, end_wait_time);
       if (!thd->killed)
         ret= mysql_cond_timedwait(&rc->cond, mutex, &timeout);
