@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 
 #include <my_global.h>
@@ -29,6 +29,9 @@
 uint trnman_active_transactions, trnman_committed_transactions,
   trnman_allocated_transactions;
 
+#ifdef WORKAROUND_GCC_4_3_2_BUG
+volatile
+#endif
 /* list of active transactions in the trid order */
 static TRN active_list_min, active_list_max;
 /* list of committed transactions in the trid order */
@@ -363,6 +366,7 @@ TRN *trnman_new_trn(WT_THD *wt)
   trn->commit_trid=  MAX_TRID;
   trn->rec_lsn= trn->undo_lsn= trn->first_undo_lsn= 0;
   trn->used_tables= 0;
+  trn->used_instances= 0;
 
   trn->locked_tables= 0;
   trn->flags= 0;
@@ -577,7 +581,6 @@ int trnman_can_read_from(TRN *trn, TrID trid)
 {
   TRN **found;
   my_bool can;
-  LF_REQUIRE_PINS(3);
 
   if (trid < trn->min_read_from)
     return 1; /* Row is visible by all transactions in the system */
@@ -618,7 +621,6 @@ int trnman_can_read_from(TRN *trn, TrID trid)
 TRN *trnman_trid_to_trn(TRN *trn, TrID trid)
 {
   TRN **found;
-  LF_REQUIRE_PINS(3);
 
   if (trid < trn->min_read_from)
     return 0; /* it's committed eons ago */
@@ -874,7 +876,7 @@ TrID trnman_get_min_safe_trid()
 {
   TrID trid;
   mysql_mutex_lock(&LOCK_trn_list);
-  trid= min(active_list_min.next->min_read_from,
+  trid= MY_MIN(active_list_min.next->min_read_from,
             global_trid_generator);
   mysql_mutex_unlock(&LOCK_trn_list);
   return trid;
