@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
 
 #ifndef MgmtSrvr_H
 #define MgmtSrvr_H
@@ -627,7 +627,7 @@ private:
   NodeId m_master_node;
 
   /**
-   * Handles the thread wich upon a 'Node is started' event will
+   * Handles the thread which upon a 'Node is started' event will
    * set the node's previous loglevel settings.
    */
   struct NdbThread* _logLevelThread;
