@@ -68,14 +68,13 @@ struct trx_extra_t {
     TABLE *table;
 };
 
-int trx_callback(
-    DB_TXN* txn,
-    iterate_row_locks_callback iterate_locks,
-    void* locks_extra,
-    void *extra) {
-
+int trx_callback(DB_TXN* txn,
+                 TOKUDB_UNUSED(iterate_row_locks_callback iterate_locks),
+                 TOKUDB_UNUSED(void* locks_extra),
+                 void* extra) {
     uint64_t txn_id = txn->id64(txn);
-    uint64_t client_id = txn->get_client_id(txn);
+    uint64_t client_id;
+    txn->get_client_id(txn, &client_id, NULL);
     uint64_t start_time = txn->get_start_time(txn);
     trx_extra_t* e = reinterpret_cast<struct trx_extra_t*>(extra);
     THD* thd = e->thd;
@@ -91,14 +90,14 @@ int trx_callback(
 }
 
 #if MYSQL_VERSION_ID >= 50600
-int trx_fill_table(THD* thd, TABLE_LIST* tables, Item* cond) {
+int trx_fill_table(THD* thd, TABLE_LIST* tables, TOKUDB_UNUSED(Item* cond)) {
 #else
-int trx_fill_table(THD* thd, TABLE_LIST* tables, COND* cond) {
+int trx_fill_table(THD* thd, TABLE_LIST* tables, TOKUDB_UNUSED(COND* cond)) {
 #endif
     TOKUDB_DBUG_ENTER("");
     int error;
 
-    tokudb_hton_initialized_lock.lock_read();
+    rwlock_t_lock_read(tokudb_hton_initialized_lock);
 
     if (!tokudb_hton_initialized) {
         error = ER_PLUGIN_IS_NOT_LOADED;
@@ -121,7 +120,7 @@ int trx_init(void* p) {
     return 0;
 }
 
-int trx_done(void* p) {
+int trx_done(TOKUDB_UNUSED(void* p)) {
     return 0;
 }
 
@@ -189,13 +188,13 @@ int lock_waits_callback(
     size_t dname_length = strlen(dname);
     table->field[2]->store(dname, dname_length, system_charset_info);
     String left_str;
-    tokudb_pretty_left_key(db, left_key, &left_str);
+    tokudb_pretty_left_key(left_key, &left_str);
     table->field[3]->store(
         left_str.ptr(),
         left_str.length(),
         system_charset_info);
     String right_str;
-    tokudb_pretty_right_key(db, right_key, &right_str);
+    tokudb_pretty_right_key(right_key, &right_str);
     table->field[4]->store(
         right_str.ptr(),
         right_str.length(),
@@ -226,14 +225,18 @@ int lock_waits_callback(
 }
 
 #if MYSQL_VERSION_ID >= 50600
-int lock_waits_fill_table(THD* thd, TABLE_LIST* tables, Item* cond) {
+int lock_waits_fill_table(THD* thd,
+                          TABLE_LIST* tables,
+                          TOKUDB_UNUSED(Item* cond)) {
 #else
-int lock_waits_fill_table(THD* thd, TABLE_LIST* tables, COND* cond) {
+int lock_waits_fill_table(THD* thd,
+                          TABLE_LIST* tables,
+                          TOKUDB_UNUSED(COND* cond)) {
 #endif
     TOKUDB_DBUG_ENTER("");
     int error;
 
-    tokudb_hton_initialized_lock.lock_read();
+    rwlock_t_lock_read(tokudb_hton_initialized_lock);
 
     if (!tokudb_hton_initialized) {
         error = ER_PLUGIN_IS_NOT_LOADED;
@@ -259,7 +262,7 @@ int lock_waits_init(void* p) {
     return 0;
 }
 
-int lock_waits_done(void *p) {
+int lock_waits_done(TOKUDB_UNUSED(void *p)) {
     return 0;
 }
 
@@ -314,7 +317,8 @@ int locks_callback(
     void* extra) {
 
     uint64_t txn_id = txn->id64(txn);
-    uint64_t client_id = txn->get_client_id(txn);
+    uint64_t client_id;
+    txn->get_client_id(txn, &client_id, NULL);
     locks_extra_t* e = reinterpret_cast<struct locks_extra_t*>(extra);
     THD* thd = e->thd;
     TABLE* table = e->table;
@@ -331,14 +335,14 @@ int locks_callback(
         table->field[2]->store(dname, dname_length, system_charset_info);
 
         String left_str;
-        tokudb_pretty_left_key(db, &left_key, &left_str);
+        tokudb_pretty_left_key(&left_key, &left_str);
         table->field[3]->store(
             left_str.ptr(),
             left_str.length(),
             system_charset_info);
 
         String right_str;
-        tokudb_pretty_right_key(db, &right_key, &right_str);
+        tokudb_pretty_right_key(&right_key, &right_str);
         table->field[4]->store(
             right_str.ptr(),
             right_str.length(),
@@ -368,14 +372,14 @@ int locks_callback(
 }
 
 #if MYSQL_VERSION_ID >= 50600
-int locks_fill_table(THD* thd, TABLE_LIST* tables, Item* cond) {
+int locks_fill_table(THD* thd, TABLE_LIST* tables, TOKUDB_UNUSED(Item* cond)) {
 #else
-int locks_fill_table(THD* thd, TABLE_LIST* tables, COND* cond) {
+int locks_fill_table(THD* thd, TABLE_LIST* tables, TOKUDB_UNUSED(COND* cond)) {
 #endif
     TOKUDB_DBUG_ENTER("");
     int error;
 
-    tokudb_hton_initialized_lock.lock_read();
+    rwlock_t_lock_read(tokudb_hton_initialized_lock);
 
     if (!tokudb_hton_initialized) {
         error = ER_PLUGIN_IS_NOT_LOADED;
@@ -398,7 +402,7 @@ int locks_init(void* p) {
     return 0;
 }
 
-int locks_done(void* p) {
+int locks_done(TOKUDB_UNUSED(void* p)) {
     return 0;
 }
 
@@ -511,15 +515,19 @@ cleanup:
 }
 
 #if MYSQL_VERSION_ID >= 50600
-int file_map_fill_table(THD* thd, TABLE_LIST* tables, Item* cond) {
+int file_map_fill_table(THD* thd,
+                        TABLE_LIST* tables,
+                        TOKUDB_UNUSED(Item* cond)) {
 #else
-int file_map_fill_table(THD* thd, TABLE_LIST* tables, COND* cond) {
+int file_map_fill_table(THD* thd,
+                        TABLE_LIST* tables,
+                        TOKUDB_UNUSED(COND* cond)) {
 #endif
     TOKUDB_DBUG_ENTER("");
     int error;
     TABLE* table = tables->table;
 
-    tokudb_hton_initialized_lock.lock_read();
+    rwlock_t_lock_read(tokudb_hton_initialized_lock);
 
     if (!tokudb_hton_initialized) {
         error = ER_PLUGIN_IS_NOT_LOADED;
@@ -541,7 +549,7 @@ int file_map_init(void* p) {
     return 0;
 }
 
-int file_map_done(void* p) {
+int file_map_done(TOKUDB_UNUSED(void* p)) {
     return 0;
 }
 
@@ -716,9 +724,13 @@ cleanup:
 }
 
 #if MYSQL_VERSION_ID >= 50600
-int fractal_tree_info_fill_table(THD* thd, TABLE_LIST* tables, Item* cond) {
+int fractal_tree_info_fill_table(THD* thd,
+                                 TABLE_LIST* tables,
+                                 TOKUDB_UNUSED(Item* cond)) {
 #else
-int fractal_tree_info_fill_table(THD* thd, TABLE_LIST* tables, COND* cond) {
+int fractal_tree_info_fill_table(THD* thd,
+                                 TABLE_LIST* tables,
+                                 TOKUDB_UNUSED(COND* cond)) {
 #endif
     TOKUDB_DBUG_ENTER("");
     int error;
@@ -726,7 +738,7 @@ int fractal_tree_info_fill_table(THD* thd, TABLE_LIST* tables, COND* cond) {
 
     // 3938: Get a read lock on the status flag, since we must
     // read it before safely proceeding
-    tokudb_hton_initialized_lock.lock_read();
+    rwlock_t_lock_read(tokudb_hton_initialized_lock);
 
     if (!tokudb_hton_initialized) {
         error = ER_PLUGIN_IS_NOT_LOADED;
@@ -749,7 +761,7 @@ int fractal_tree_info_init(void* p) {
     return 0;
 }
 
-int fractal_tree_info_done(void* p) {
+int fractal_tree_info_done(TOKUDB_UNUSED(void* p)) {
     return 0;
 }
 
@@ -1010,12 +1022,12 @@ cleanup:
 int fractal_tree_block_map_fill_table(
     THD* thd,
     TABLE_LIST* tables,
-    Item* cond) {
+    TOKUDB_UNUSED(Item* cond)) {
 #else
 int fractal_tree_block_map_fill_table(
     THD* thd,
     TABLE_LIST* tables,
-    COND* cond) {
+    TOKUDB_UNUSED(COND* cond)) {
 #endif
     TOKUDB_DBUG_ENTER("");
     int error;
@@ -1023,7 +1035,7 @@ int fractal_tree_block_map_fill_table(
 
     // 3938: Get a read lock on the status flag, since we must
     // read it before safely proceeding
-    tokudb_hton_initialized_lock.lock_read();
+    rwlock_t_lock_read(tokudb_hton_initialized_lock);
 
     if (!tokudb_hton_initialized) {
         error = ER_PLUGIN_IS_NOT_LOADED;
@@ -1046,7 +1058,7 @@ int fractal_tree_block_map_init(void* p) {
     return 0;
 }
 
-int fractal_tree_block_map_done(void *p) {
+int fractal_tree_block_map_done(TOKUDB_UNUSED(void *p)) {
     return 0;
 }
 
@@ -1085,7 +1097,7 @@ ST_FIELD_INFO background_job_status_field_info[] = {
     {"scheduler", 32, MYSQL_TYPE_STRING, 0, 0, NULL, SKIP_OPEN_TABLE },
     {"scheduled_time", 0, MYSQL_TYPE_DATETIME, 0, 0, NULL, SKIP_OPEN_TABLE },
     {"started_time", 0, MYSQL_TYPE_DATETIME, 0, MY_I_S_MAYBE_NULL, NULL, SKIP_OPEN_TABLE },
-    {"status", 256, MYSQL_TYPE_STRING, 0, MY_I_S_MAYBE_NULL, SKIP_OPEN_TABLE },
+    {"status", 1024, MYSQL_TYPE_STRING, 0, MY_I_S_MAYBE_NULL, NULL, SKIP_OPEN_TABLE },
     {NULL, 0, MYSQL_TYPE_NULL, 0, 0, NULL, SKIP_OPEN_TABLE}
 };
 
@@ -1095,15 +1107,7 @@ struct background_job_status_extra {
 };
 
 void background_job_status_callback(
-    uint64_t id,
-    const char* database_name,
-    const char* table_name,
-    const char* type,
-    const char* params,
-    const char* status,
-    bool user_scheduled,
-    time_t scheduled_time,
-    time_t started_time,
+    tokudb::background::job_manager_t::job_t* job,
     void* extra) {
 
     background_job_status_extra* e =
@@ -1111,24 +1115,33 @@ void background_job_status_callback(
 
     THD* thd = e->thd;
     TABLE* table = e->table;
+    const char* tmp = NULL;
 
-    table->field[0]->store(id, false);
-    table->field[1]->store(
-        database_name,
-        strlen(database_name),
-        system_charset_info);
-    table->field[2]->store(table_name, strlen(table_name), system_charset_info);
-    table->field[3]->store(type, strlen(type), system_charset_info);
-    table->field[4]->store(params, strlen(params), system_charset_info);
-    if (user_scheduled)
+    table->field[0]->store(job->id(), false);
+
+    tmp = job->database();
+    table->field[1]->store(tmp, strlen(tmp),  system_charset_info);
+
+    tmp = job->table();
+    table->field[2]->store(tmp, strlen(tmp),  system_charset_info);
+
+    tmp = job->type();
+    table->field[3]->store(tmp, strlen(tmp),  system_charset_info);
+
+    tmp = job->parameters();
+    table->field[4]->store(tmp, strlen(tmp),  system_charset_info);
+
+    if (job->user_scheduled())
         table->field[5]->store("USER", strlen("USER"), system_charset_info);
     else
         table->field[5]->store("AUTO", strlen("AUTO"), system_charset_info);
 
-    field_store_time_t(table->field[6], scheduled_time);
-    field_store_time_t(table->field[7], started_time);
-    if (status[0] != '\0') {
-        table->field[8]->store(status, strlen(status), system_charset_info);
+    field_store_time_t(table->field[6], job->scheduled_time());
+    field_store_time_t(table->field[7], job->started_time());
+
+    tmp = job->status();
+    if (tmp && tmp[0] != '\0') {
+        table->field[8]->store(tmp, strlen(tmp), system_charset_info);
         table->field[8]->set_notnull();
     } else {
         table->field[8]->store(NULL, 0, system_charset_info);
@@ -1151,15 +1164,15 @@ int report_background_job_status(TABLE *table, THD *thd) {
 }
 
 #if MYSQL_VERSION_ID >= 50600
-int background_job_status_fill_table(THD *thd, TABLE_LIST *tables, Item *cond) {
+int background_job_status_fill_table(THD *thd, TABLE_LIST *tables, TOKUDB_UNUSED(Item *cond)) {
 #else
-int background_job_status_fill_table(THD *thd, TABLE_LIST *tables, COND *cond) {
+int background_job_status_fill_table(THD *thd, TABLE_LIST *tables, TOKUDB_UNUSED(COND *cond)) {
 #endif
     TOKUDB_DBUG_ENTER("");
     int error;
     TABLE* table = tables->table;
 
-    tokudb_hton_initialized_lock.lock_read();
+    rwlock_t_lock_read(tokudb_hton_initialized_lock);
 
     if (!tokudb_hton_initialized) {
         error = ER_PLUGIN_IS_NOT_LOADED;
@@ -1181,7 +1194,7 @@ int background_job_status_init(void* p) {
     return 0;
 }
 
-int background_job_status_done(void* p) {
+int background_job_status_done(TOKUDB_UNUSED(void* p)) {
     return 0;
 }
 
